/*************************************************************************
**
** bl_Serial.c
** Win32 module by Martin Sikstrm <e93_msi@e.kth.se>
** Copyright (c) 1995,1996,1999 Daniel Kahlin <tlr@stacken.kth.se>
**
******/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef WIN32_VERSION
#include <windows.h>
#endif /* WIN32_VERSION */

#include "mach_include.h"
#include "Block.h"
#include "main.h"
#include "mach.h"

#define SER_BAUDRATE 38400
#define SER_BREAKLENGTH 250000
#define SER_BITSPERCHAR 8
#define SER_STOPBITS 2
#define SER_QUANTUM 256
#define SERBUFFERSIZE 512
#define SERDEVICE "serial.device"
#define SERUNITNUM 0

#define ERRBUFFERSIZE 256

/*** serial globals ***/
HANDLE          serhandle=NULL;
COMMPROP        serprop;
DCB             serdcb;
COMMTIMEOUTS	sertimeout;
char            errorbuffer[ERRBUFFERSIZE];
int             quitflag;

/* Function prototypes (all others are declared in Block.h */
void breakhandler(void);
BOOL InitParams(int32_t baudrate);
void dump16(u_int8_t *ptr, int size, char *str);

/*
** Write buffer to serport   (Size==-1) => zero terminated string
*/
void SerWrite(APTR Buffer, int32_t Size)
{
    DWORD byteswritten;

#ifdef ECHO_FUNCTION_CALL
    printf ("SerWrite(%p, %ld);\n", Buffer, Size);
#endif

    if(Size < 0)
	Size = strlen(Buffer);

    if (serhandle)
    {
	if (!WriteFile(serhandle, Buffer, Size, &byteswritten, NULL))
	{
	    if (GetLastError() != ERROR_IO_PENDING)
	    {
		puts(GetSerialError(GetLastError()));
		panic("WriteFile failed");
	    }
	    if (byteswritten != (unsigned) Size)
	    {
		panic("Requested and written bytes do not match");
	    }
	}
    }
}


/*
** Read from serport to buffer (timeout==-1 => no timeout)
*/
int SerRead(APTR Buffer, u_int32_t Size, int32_t timeout)
{
    DWORD bytesread;

#ifdef ECHO_FUNCTION_CALL
    printf ("SerRead(%p, %ld, %ld);\n", Buffer, Size, timeout);
#endif

    sertimeout.ReadIntervalTimeout=0;
    sertimeout.ReadTotalTimeoutMultiplier=0;
    sertimeout.WriteTotalTimeoutMultiplier=0;
    sertimeout.WriteTotalTimeoutConstant=0;
    
    if (timeout<0)
	/* Do read without timeout in chunks of 2 seconds */
	sertimeout.ReadTotalTimeoutConstant=2000;
    else
	/* timeout is given in seconds */
	sertimeout.ReadTotalTimeoutConstant=timeout*1000;

    if (serhandle) {
	do {
	    SetCommTimeouts(serhandle, &sertimeout);
	    
	    if (!ReadFile(serhandle, Buffer, Size, &bytesread, NULL))
		/* There was an error (timeout is not an error) */
		panic(GetSerialError(GetLastError()));
	
	    if (bytesread == Size)
		/* There was no timeout */
		return (0);

	    /* In case of read without timeout, go back and wait for
	       the remaining bytes */
	    Size -= bytesread;
	    Buffer += bytesread;
	} while (timeout < 0 && !quitflag);
    }
    
    return (1);
}

/*
** Allocate all serial stuffs
*/
STRPTR CreateSerial()
{
#ifdef ECHO_FUNCTION_CALL
    printf ("CreateSerial();\n");
#endif

    /* Byt ut "COM2" mot o5config.serialdevice eller nt */
    serhandle=CreateFile(o5config.serdevice, GENERIC_READ | GENERIC_WRITE,
			 0, NULL, OPEN_EXISTING, 
			 FILE_ATTRIBUTE_NORMAL,
			 NULL);
    if (serhandle==INVALID_HANDLE_VALUE) {
	serhandle=NULL;
	return (GetSerialError(GetLastError()));
    }
    
    if (!SetupComm(serhandle, SERBUFFERSIZE, SERBUFFERSIZE))
	return (GetSerialError(GetLastError()));

    if (!SetCommMask(serhandle, EV_RXCHAR))
	return (GetSerialError(GetLastError()));

    if (!InitParams(SER_BAUDRATE))
	return (GetSerialError(GetLastError()));

    quitflag=0;

    return(NULL);
}


/*
** clear
*/
void ClearSerial(void)
{
#ifdef ECHO_FUNCTION_CALL
    printf ("ClearSerial();\n");
#endif

    if (serhandle) {
	FlushFileBuffers(serhandle);
	PurgeComm(serhandle, PURGE_TXCLEAR | PURGE_RXCLEAR);
	ClearCommBreak(serhandle);
    }
}


/*
** Deallocate all serial stuff
*/
void DeleteSerial()
{
#ifdef ECHO_FUNCTION_CALL
    printf ("DeleteSerial();\n");
#endif

    if (serhandle) {
	quitflag=1;
	ClearSerial();
	CloseHandle(serhandle);
	serhandle=NULL;
    }
}

    
/*
** Set Baud rate etc...
*/
void Setbaud(u_int32_t baudrate)
{
#ifdef ECHO_FUNCTION_CALL
    printf ("Setbaud(%ld)\n", baudrate);
#endif

    if (serhandle) {
	serdcb.BaudRate=baudrate;
	SetCommState(serhandle, &serdcb);
    }
}


/*
** Sendbreak
*/
void SendBreak(void)
{
#ifdef ECHO_FUNCTION_CALL
    printf ("Sendbreak();");
#endif

    if (serhandle)
	SetCommBreak(serhandle);
}


/*
** Return string for serial error
*/
STRPTR GetSerialError(int32_t Error)
{
    int offs;
    char *ptr;

#ifdef ECHO_FUNCTION_CALL
    printf ("GetSerialError(%ld);\n", Error);
#endif
    
    offs = sprintf (errorbuffer, "%s: ", o5config.serdevice);
    FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, Error, 0,
		  errorbuffer+offs, ERRBUFFERSIZE-1-offs, NULL);

    /* FormatMessage() usually puts a ".\r\n" at the end of the string,
       which will look ugly when panic() puts an exclamation mark after. */
    if ((ptr=strrchr(errorbuffer, '.')))
	*ptr='\0';
    return (errorbuffer);
}


BOOL InitParams(int32_t baudrate)
{
#ifdef ECHO_FUNCTION_CALL
    printf ("InitParams(%ld)\n", baudrate);
#endif

    if (serhandle) {
	if (!(GetCommProperties(serhandle, &serprop)))
	    return FALSE;
	if (!(GetCommState(serhandle, &serdcb)))
	    return FALSE;

	serdcb.BaudRate=baudrate;
	serdcb.fBinary=TRUE;
	serdcb.fParity=TRUE;
	serdcb.fOutxCtsFlow=FALSE;
	serdcb.fOutxDsrFlow=FALSE;
	serdcb.fDtrControl=DTR_CONTROL_DISABLE;
	serdcb.fOutX=FALSE;
	serdcb.fInX=FALSE;
	serdcb.fErrorChar=FALSE;
	serdcb.fNull=FALSE;
	serdcb.fRtsControl=RTS_CONTROL_DISABLE;
	serdcb.fAbortOnError=FALSE;
	serdcb.ByteSize=SER_BITSPERCHAR;
	serdcb.Parity=NOPARITY;
	serdcb.StopBits=SER_STOPBITS;

	if (!SetCommState(serhandle, &serdcb))
	    return FALSE;
    }
    return TRUE;
}

/*************************************************************************
**
** microwait.
**
** Note: the number of microseconds to sleep is rounded to milliseconds.
**
******/
void MicroWait(u_int32_t microsecs)
{
#ifdef ECHO_FUNCTION_CALL
    printf ("MicroWait(%ld);\n", microsecs);
#endif

    Sleep((microsecs+500)/1000);
}


void dump16(u_int8_t *ptr, int size, char *str)
{
    int i;
    char hexarr[]="0123456789ABCDEF";
    char buf[256], *b;

    strcpy(buf, str);

    while(size>0)
    {
        b=buf+strlen(str);

    	for (i=0; i<min(size,16); i++)
    	{
	    *b++=hexarr[*ptr>>4];
	    *b++=hexarr[*(ptr++)&15];
	    *b++=' ';
        }
        *b='\0';
        puts(buf);
        size-=16;
    }
}

